 	

 

"Black-Scholes" in Multiple Languages

January 2008: After studying the literature (something many of the famous academics themselves obviously not have done properly) it is obvious that we option traders never have used the Black-Scholes-Merton formula in practice.( see also article in Frobes ) Only if you use close to continuous time delta hedging to remove close to all the risk all the time you are actually using the Black-Scholes (or the Black-Scholes-Merton) version of the option formula. The only problem this is impossible in practice.

If you remove most risk by hedging options with options, get immune for blow up risk by the way you construct your option portfolio then you are using the traders formula/method that was discovered before Black-Scholes-Merton by a series of traders and researchers, the first contribution form Bachelier 1900 and the last by Thorp 1969, so this is why we think it should be called the Bachelier-Thorp formula. In practice you can remove risk with discrete delta hedging (known long before Black-Scholes and Merton), but you can not remove enough risk to argue for risk-neutral valuation (and this is the main argument of Black-Scholes-Merton). See Chapter 2 in my book Derivatives Models on Models for a detailed discussion on how to hedge options in practice.

You naturally know the so called "Black-Scholes-Merton" option formula, that actualy not is the Black-Scholes-Merton formula (BSM was a theoretical hedging argument related to risk-neutral valuation), but in how many languages? Just like me I guess you speak Norwegian, French, Russian, English, Swedish and Danish, but what about really interesting languages like (now in more than 30 languages):

Autoit, Fortress, Lua, APL, SAS, Mathcad, J, MEL, Postscript, VB.NET, Clean, Ruby, Lisp, Prolog, PL/SQL, LyME, ColdFusion, K, C#, HP48, Transact SQL, O'Caml, Rebol, Real Basic, Icon, Squeak, Haskell, JAVA , JavaScript, VBA, C++, Perl, Maple, Mathematica, Matlab, S-Plus, IDL, Pascal, Python, Fortran, Scheme, PHP, GNU, gnuplot...

If you have implemented Black-Scholes in another language I would be happy to get a copy of your source code to put it on this page!

(in that case try to use the same symbols and the setup as below)

 

 


In the different implementations below we will use the symbols:

S= Stock price

X=Strike price

T=Years to maturity

r= Risk-free rate

v=Volatility

 

 

Black-Scholes Directly in a Excel Sheet
("keep it simple stupid")

If you are afraid of programing languages you can start with doing Black-Scholes directly in an Excel sheet, just type in what you see below. If you are using the Norwegian or French version of Excel you have to do some translation yourself:

Are you too lazy to type in what you see above? Okay download me here

 

 

Black-Scholes in Visual Basic

By Espen Gaarder Haug

Visual Basic: easy to program but quite slow!

'// The Black and Scholes (1973) Stock option formula
Public Function BlackScholes(CallPutFlag As String, S As Double, X _
As Double, T As Double, r As Double, v As Double) As Double

Dim d1 As Double, d2 As Double

d1 = (Log(S / X) + (r + v ^ 2 / 2) * T) / (v * Sqr(T))
d2 = d1 - v * Sqr(T)
If CallPutFlag = "c" Then
BlackScholes = S * CND(d1) - X * Exp(-r * T) * CND(d2)
ElseIf CallPutFlag = "p" Then
BlackScholes = X * Exp(-r * T) * CND(-d2) - S * CND(-d1)
End If
End Function

'// The cumulative normal distribution function
Public Function CND(X As Double) As Double

Dim L As Double, K As Double
Const a1 = 0.31938153: Const a2 = -0.356563782: Const a3 = 1.781477937:
Const a4 = -1.821255978: Const a5 = 1.330274429

L = Abs(X)
K = 1 / (1 + 0.2316419 * L)
CND = 1 - 1 / Sqr(2 * Application.Pi()) * Exp(-L ^ 2 / 2) * (a1 * K + a2 * K ^ 2 + a3 * K ^ 3 + a4 * K ^ 4 + a5 * K ^ 5)

If X < 0 Then
CND = 1 - CND
End If
End Function

 

Black-Scholes in

By Espen Gaarder Haug

C++: a bit harder than most other languages but very fast and powerful. After my opinion the Rolls Royce computer language for
 mathematical models where you need speed (for closed form solutions like Blacks-Scholes you are naturally doing fine in almost any language,  but when it comes to large scale Monte Carlo C++ is really a plus).

#ifndef Pi
#define Pi 3.141592653589793238462643
#endif

// The Black and Scholes (1973) Stock option formula
double BlackScholes(char CallPutFlag, double S, double X, double T, double r, double v)
{
double d1, d2;


d1=(log(S/X)+(r+v*v/2)*T)/(v*sqrt(T));
d2=d1-v*sqrt(T);

if(CallPutFlag == 'c')
return S *CND(d1)-X * exp(-r*T)*CND(d2);
else if(CallPutFlag == 'p')
return X * exp(-r * T) * CND(-d2) - S * CND(-d1);
}

// The cumulative normal distribution function
double CND( double X )
{

double L, K, w ;

double const a1 = 0.31938153, a2 = -0.356563782, a3 = 1.781477937;
double const a4 = -1.821255978, a5 = 1.330274429;

L = fabs(X);
K = 1.0 / (1.0 + 0.2316419 * L);
w = 1.0 - 1.0 / sqrt(2 * Pi) * exp(-L *L / 2) * (a1 * K + a2 * K *K + a3 * pow(K,3) + a4 * pow(K,4) + a5 * pow(K,5));

if (X < 0 ){
w= 1.0 - w;
}
return w;
} 
